@@ -18,6 +18,19 @@
 package org.apache.spark.util
 
 private[spark] object SparkExitCode {
+
+  /** Successful termination. */
+  val EXIT_SUCCESS = 0
+
+  /** Failed termination. */
+  val EXIT_FAILURE = 1
+
+  /** Exception indicate invalid usage of some shell built-in command. */
+  val ERROR_MISUSE_SHELL_BUILTIN = 2
+
+  /** Exception appears when the computer cannot find the specified path. */
+  val ERROR_PATH_NOT_FOUND = 3
+
   /** The default uncaught exception handler was reached. */
   val UNCAUGHT_EXCEPTION = 50
 
@@ -29,4 +42,6 @@ private[spark] object SparkExitCode {
       OutOfMemoryError. */
   val OOM = 52
 
+  /** Exception indicate command not found. */
+  val ERROR_COMMAND_NOT_FOUND = 127
 }
