@@ -19,6 +19,7 @@ package org.apache.spark.sql
 
 import java.io.{Externalizable, ObjectInput, ObjectOutput}
 import java.sql.{Date, Timestamp}
+import java.util.UUID
 
 import scala.util.Random
 
@@ -269,6 +270,13 @@ class DatasetSuite extends QueryTest
       (ClassData("one", 2), 1L), (ClassData("two", 3), 1L))
   }
 
+  test("SPARK-45896: seq of option of seq") {
+    val ds = Seq(DataSeqOptSeq(Seq(Some(Seq(0))))).toDS()
+    checkDataset(
+      ds,
+      DataSeqOptSeq(Seq(Some(List(0)))))
+  }
+
   test("select") {
     val ds = Seq(("a", 1), ("b", 2), ("c", 3)).toDS()
     checkDataset(
@@ -2461,6 +2469,45 @@ class DatasetSuite extends QueryTest
     )
     assert(result == expected)
   }
+
+  test("SPARK-45282: Coaleasced shuffle read is not compatible with hash partitioning") {
+    withSQLConf(
+        SQLConf.ADVISORY_PARTITION_SIZE_IN_BYTES.key  -> "33554432",
+        SQLConf.COALESCE_PARTITIONS_PARALLELISM_FIRST.key -> "false",
+        SQLConf.CAN_CHANGE_CACHED_PLAN_OUTPUT_PARTITIONING.key -> "true") {
+      val data = (1 to 1000000).toDS().map(i => UUID.randomUUID().toString).persist()
+
+      val left = data.map(k => (k, 1))
+      val right = data.map(k => (k, k))
+
+      val left1 = left
+        .toDF("key", "value1")
+        .repartition(col("key"))
+        .persist()
+      left1.count()
+
+      val right1 = right
+        .toDF("key", "value2")
+        .repartition(col("key"))
+        .persist()
+
+      val join = left1.join(right1, "key")
+
+      assert(join.count() == 1000000)
+    }
+  }
+
+  test("SPARK-47385: Tuple encoder with Option inputs") {
+    implicit val enc: Encoder[(SingleData, Option[SingleData])] =
+      Encoders.tuple(Encoders.product[SingleData], Encoders.product[Option[SingleData]])
+
+    val input = Seq(
+      (SingleData(1), Some(SingleData(1))),
+      (SingleData(2), None)
+    )
+    val ds = spark.createDataFrame(input).as[(SingleData, Option[SingleData])]
+    checkDataset(ds, input: _*)
+  }
 }
 
 class DatasetLargeResultCollectingSuite extends QueryTest
@@ -2532,6 +2579,8 @@ case class ClassNullableData(a: String, b: Integer)
 case class NestedStruct(f: ClassData)
 case class DeepNestedStruct(f: NestedStruct)
 
+case class DataSeqOptSeq(a: Seq[Option[Seq[Int]]])
+
 /**
  * A class used to test serialization using encoders. This class throws exceptions when using
  * Java serialization -- so the only way it can be "serialized" is through our encoders.
